/* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* servicesman services actions
 * Copyright (C) 2001 Kevin Vandersloot
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <config.h>
#include <errno.h>
#include <glib/gi18n.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "servicesactions.h"
#include "application.h"
#include "servicestable.h"
#include "servicesdialogs.h"
#include <servicesutil.h>

static void start_single_service(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
    const struct ServicesActionArgs *const args = static_cast<ServicesActionArgs *>(data);

    ServicesInfo *info = NULL;
    gint error;
    int saved_errno;
    gchar *error_msg;
    GtkMessageDialog *dialog;

    gtk_tree_model_get (model, iter, SERVICES_COL_POINTER, &info,
    -1);

    GError * err = NULL;
    start_service_util(info->unit_path, &err);
}

static void stop_single_service(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
    const struct ServicesActionArgs *const args = static_cast<ServicesActionArgs *>(data);

    ServicesInfo *info = NULL;
    gint error;
    int saved_errno;
    gchar *error_msg;
    GtkMessageDialog *dialog;

    gtk_tree_model_get (model, iter, SERVICES_COL_POINTER, &info,
    -1);

    GError * err = NULL;
    stop_service_util(info->unit_path, &err);
}

static void restart_single_service(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
    const struct ServicesActionArgs *const args = static_cast<ServicesActionArgs *>(data);

    ServicesInfo *info = NULL;
    gint error;
    int saved_errno;
    gchar *error_msg;
    GtkMessageDialog *dialog;

    gtk_tree_model_get (model, iter, SERVICES_COL_POINTER, &info,
    -1);

    GError * err = NULL;
    restart_service_util(info->unit_path, &err);
}

void do_process(GsmApplication * app, int sig) {

    switch(sig) {
        case SERVICESMAN_ACTION_START:
            start_service ( app, sig);
            break;
        case SERVICESMAN_ACTION_STOP:
            stop_service ( app, sig);
            break;
        case SERVICESMAN_ACTION_RESTART:
            restart_service ( app, sig);
            break;
        case SERVICESMAN_ACTION_ENABLE:
            enable_service ( app, sig);
            break;
        case SERVICESMAN_ACTION_DISABLE:
            disable_service ( app, sig);
            break;
        default:
            break;
    }

}

void stop_service(GsmApplication *app, int sig) {
    struct ServicesActionArgs args = {app, sig};
    servicestable_freeze(app);
    gtk_tree_selection_selected_foreach(app->services_selection, stop_single_service,
                                                        & args);
    servicestable_thaw(app);
    servicestable_update(app);
}

void start_service(GsmApplication *app, int sig) {
    struct ServicesActionArgs args = {app, sig};
    servicestable_freeze(app);
    gtk_tree_selection_selected_foreach(app->services_selection, start_single_service,
                                        & args);
    servicestable_thaw(app);
    servicestable_update(app);
}

void restart_service(GsmApplication *app, int sig) {
    struct ServicesActionArgs args = {app, sig};
    servicestable_freeze(app);
    gtk_tree_selection_selected_foreach(app->services_selection, restart_single_service,
                                        & args);
    servicestable_thaw(app);
    servicestable_update(app);
}

void enable_service(GsmApplication *app, int sig) {

}

void disable_service(GsmApplication *app, int sig) {

}