//
// Created by cuilei on 2020/3/26.
//

#ifndef GNOME_SYSTEM_MONITOR_3_32_1_SERVICESUTIL_H
#define GNOME_SYSTEM_MONITOR_3_32_1_SERVICESUTIL_H

#include <interface.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <systemd/sd-bus.h>

#define _cleanup_(f) __attribute__((cleanup(f)))

static inline void freep(void *p) {
    free(*(void**) p);
}

#define _cleanup_free_ _cleanup_(freep)

#define _cleanup_strv_free_ _cleanup_(strv_freep)

#define laccess(path, mode) faccessat(AT_FDCWD, (path), (mode), AT_SYMLINK_NOFOLLOW)

#define streq(a,b) (strcmp((a),(b)) == 0)
#define strcaseeq(a,b) (strcasecmp((a),(b)) == 0)
#define SYNTHETIC_ERRNO(num)                (1 << 30 | (num))
#define LIST_HEAD(t,name)                                               \
        t *name
#define strjoin(a, ...) strjoin_real((a), __VA_ARGS__, NULL)
#define _unlikely_(x) (__builtin_expect(!!(x), 0))
#define service_new(t, n) ((t*) malloc_multiply(sizeof(t), (n)))

#define GREEDY_REALLOC(array, allocated, need)                          \
        greedy_realloc((void**) &(array), &(allocated), (need), sizeof((array)[0]))
#define _likely_(x) (__builtin_expect(!!(x), 1))


typedef enum BusFocus {
    BUS_FULL,      /* The full bus indicated via --system or --user */
    BUS_MANAGER,   /* The manager itself, possibly directly, possibly via the bus */
    _BUS_FOCUS_MAX
} BusFocus;

typedef enum BusTransport {
    BUS_TRANSPORT_LOCAL,
    BUS_TRANSPORT_REMOTE,
    BUS_TRANSPORT_MACHINE,
    _BUS_TRANSPORT_MAX,
    _BUS_TRANSPORT_INVALID = -1
} BusTransport;

typedef enum OutputMode {
    OUTPUT_SHORT,
    OUTPUT_SHORT_FULL,
    OUTPUT_SHORT_ISO,
    OUTPUT_SHORT_ISO_PRECISE,
    OUTPUT_SHORT_PRECISE,
    OUTPUT_SHORT_MONOTONIC,
    OUTPUT_SHORT_UNIX,
    OUTPUT_VERBOSE,
    OUTPUT_EXPORT,
    OUTPUT_JSON,
    OUTPUT_JSON_PRETTY,
    OUTPUT_JSON_SSE,
    OUTPUT_JSON_SEQ,
    OUTPUT_CAT,
    OUTPUT_WITH_UNIT,
    _OUTPUT_MODE_MAX,
    _OUTPUT_MODE_INVALID = -1
} OutputMode;

enum UnitFileScope {
    UNIT_FILE_SYSTEM,
    UNIT_FILE_GLOBAL,
    UNIT_FILE_USER,
    _UNIT_FILE_SCOPE_MAX,
    _UNIT_FILE_SCOPE_INVALID = -1
};

typedef enum PagerFlags {
    PAGER_DISABLE     = 1 << 0,
    PAGER_JUMP_TO_END = 1 << 1,
} PagerFlags;

enum bus_state {
    BUS_UNSET,
    BUS_WATCH_BIND,      /* waiting for the socket to appear via inotify */
    BUS_OPENING,         /* the kernel's connect() is still not ready */
    BUS_AUTHENTICATING,  /* we are currently in the "SASL" authorization phase of dbus */
    BUS_HELLO,           /* we are waiting for the Hello() response */
    BUS_RUNNING,
    BUS_CLOSING,
    BUS_CLOSED,
    _BUS_STATE_MAX,
};

struct sd_bus_message {
    const char *path;
    const char *interface;
    sd_bus_error error;
    int64_t priority;
    struct bus_header *header;
    struct bus_body_part *body_end;
    struct bus_body_part *cached_rindex_part;
};

typedef struct UnitInfo {
    const char *machine;
    char *id;
    const char *description;
    const char *load_state;
    const char *active_state;
    const char *sub_state;
    const char *following;
    const char *unit_path;
    uint32_t job_id;
    const char *job_type;
    const char *job_path;
} UnitInfo;

struct sd_bus {
    enum bus_state state;
    LIST_HEAD(struct filter_callback, filter_callbacks);
    char *address;
    char *label;
    sd_event *event;
    char *description;
    LIST_HEAD(sd_bus_slot, slots);
    LIST_HEAD(sd_bus_track, tracks);
};

int get_list_units(ServicesList ** unit_infos,
                   sd_bus_message **_reply,
                   std::string tag);



void start_service_util(std::string unit_path,
               GError** error);

void stop_service_util(std::string unit_path,
              GError** error);

void restart_service_util(std::string unit_path,
               GError** error);

//int enable_unit(std::string unit_path,
//                GError** error);
//
//int disable_unit(std::string unit_path,
//                 GError** error);

#endif //GNOME_SYSTEM_MONITOR_3_32_1_SERVICESUTIL_H
